from .authentication import Authentication
from .firewall import Firewall
from .firmware import Firmware
from .interfaces import Interfaces
from .unauthorized import Unauthorized
from .users import Users
from .wireguard import WireGuard

__all__ = [
    "Authentication",
    "Firewall",
    "Firmware",
    "Interfaces",
    "Unauthorized",
    "Users",
    "WireGuard",
]



from ..._client import APIClient
from ...endpoints import Authentication, Firewall, Firmware, Interfaces, Unauthorized, Users, WireGuard


class Router():
    def __init__(self, base_url: str) -> None:
        self._client = APIClient(base_url)

        self.authentication = Authentication(self._client)
        self.firewall = Firewall(self._client)
        self.firmware = Firmware(self._client)
        self.interfaces = Interfaces(self._client)
        self.unauthorized = Unauthorized(self._client)
        self.users = Users(self._client)
        self.wireguard = WireGuard(self._client)

        self._endpoints = [
            self.authentication,
            self.firewall,
            self.firmware,
            self.interfaces,
            self.unauthorized,
            self.users,
            self.wireguard
        ]

    def __getattr__(self, attr: str):
        for endpoint in self._endpoints:
            if hasattr(endpoint, attr):
                return getattr(endpoint, attr)

        raise AttributeError(
            f"'{self.__class__.__name__}' object has no attribute '{attr}'"
        )



import httpx
from . import __version__


class APIClient:
    def __init__(self, base_url: str, timeout: float = 10.0) -> None:
        self._client = httpx.Client(
            base_url=base_url.rstrip("/") + "/api",
            timeout=timeout,
            headers={"User-Agent": f"pytonika/{__version__}"},
        )

    def set_token(self, token: str) -> None:
        self._client.headers["Authorization"] = f"Bearer {token}"

    def clear_token(self) -> None:
        self._client.headers.pop("Authorization", None)

    def get(self, endpoint: str, params: dict[str, object] | None = None) -> dict[str, object]:
        return self._client.get(endpoint, params=params).json()

    def post(self, endpoint: str, data: dict[str, object] | None = None) -> dict[str, object]:
        return self._client.post(endpoint, json=data).json()

    def put(self, endpoint: str, data: dict[str, object] | None = None) -> dict[str, object]:
        return self._client.put(endpoint, json=data).json()

    def delete(self, endpoint: str, data: dict[str, object] | None = None) -> dict[str, object]:
        return self._client.delete(endpoint, json=data).json()



from .routers import *
from .gateways import *
from .access_points import *
from .switches import *

__all__ = [
    "Router",
    "RUT200",
    "RUT240",
    "RUT241",
    "RUT956",
    "RUTX08",
    "RUTX50",
    "Gateway",
    "AccessPoint",
    "Switch",
]



from .._client import APIClient


class Unauthorized:
    def __init__(self, api_client: APIClient) -> None:
        self._api_client = api_client

    def get_unauthorized_status(self) -> dict[str, object]:
        endpoint = "/unauthorized/status"

        return self._api_client.get(endpoint)



from .__version__ import __title__, __description__, __version__
from .devices import *

__all__ = [
    "__title__",
    "__description__",
    "__version__",
    "Router",
    "RUT200",
    "RUT240",
    "RUT241",
    "RUT956",
    "RUTX08",
    "RUTX50",
    "Gateway",
    "AccessPoint",
    "Switch",
]



from .._client import APIClient


class Authentication:
    def __init__(self, api_client: APIClient) -> None:
        self._api_client = api_client

    def login(self, username: str, password: str) -> dict[str, object]:
        endpoint = "/login"

        data = {
            "username": username,
            "password": password
        }

        response = self._api_client.post(endpoint, data=data)

        token = response["data"].get("token")

        if token:
            self._api_client.set_token(token)

        return response

    def logout(self) -> dict[str, object]:
        endpoint = "/logout"

        response = self._api_client.post(endpoint)

        self._api_client.clear_token()

        return response

    def get_session_status(self) -> dict[str, object]:
        endpoint = "/session/status"

        return self._api_client.get(endpoint)



from .._client import APIClient


class Users:
    def __init__(self, api_client: APIClient) -> None:
        self._api_client = api_client

    def get_users_config(self) -> dict[str, object]:
        endpoint = "/users/config"

        return self._api_client.get(endpoint)

    def create_users_config(self, config: dict[str, object]) -> dict[str, object]:
        endpoint = "/users/config"

        data = {"data": config}

        return self._api_client.post(endpoint, data=data)

    def update_users_config(self, config: list[dict[str, object]]) -> dict[str, object]:
        endpoint = "/users/config"

        data = {"data": config}

        return self._api_client.put(endpoint, data=data)

    def delete_users_config(self, config: list[str]) -> dict[str, object]:
        endpoint = "/users/config"

        data = {"data": config}

        return self._api_client.delete(endpoint, data=data)

    def get_users_config_by_id(self, user_id: str) -> dict[str, object]:
        endpoint = f"/users/config/{user_id}"

        return self._api_client.get(endpoint)

    def update_users_config_by_id(self, user_id: str, config: dict[str, object]) -> dict[str, object]:
        endpoint = f"/users/config/{user_id}"

        data = {"data": config}

        return self._api_client.put(endpoint, data=data)

    def delete_users_config_by_id(self, user_id: str) -> dict[str, object]:
        endpoint = f"/users/config/{user_id}"

        return self._api_client.delete(endpoint)



from .router import Router


class RUT956(Router):
    def __init__(self, base_url: str) -> None:
        super().__init__(base_url)

        self._endpoints.extend([])

    def __getattr__(self, attr: str):
        for endpoint in self._endpoints:
            if hasattr(endpoint, attr):
                return getattr(endpoint, attr)

        return super().__getattr__(attr)



from .._client import APIClient


class WireGuard:
    def __init__(self, api_client: APIClient) -> None:
        self._api_client = api_client

    def get_wireguard_config(self) -> dict[str, object]:
        endpoint = "/wireguard/config"

        return self._api_client.get(endpoint)

    def create_wireguard_config(self, config: dict[str, object]) -> dict[str, object]:
        endpoint = "/wireguard/config"

        data = {"data": config}

        return self._api_client.post(endpoint, data=data)

    def update_wireguard_config(self, config: list[dict[str, object]]) -> dict[str, object]:
        endpoint = "/wireguard/config"

        data = {"data": config}

        return self._api_client.put(endpoint, data=data)

    def delete_wireguard_config(self, config: list[str]) -> dict[str, object]:
        endpoint = "/wireguard/config"

        data = {"data": config}

        return self._api_client.delete(endpoint, data=data)

    def get_wireguard_config_by_id(self, wireguard_id: str) -> dict[str, object]:
        endpoint = f"/wireguard/config/{wireguard_id}"

        return self._api_client.get(endpoint)

    def update_wireguard_config_by_id(self, wireguard_id: str, config: dict[str, object]) -> dict[str, object]:
        endpoint = f"/wireguard/config/{wireguard_id}"

        data = {"data": config}

        return self._api_client.put(endpoint, data=data)

    def delete_wireguard_config_by_id(self, wireguard_id: str) -> dict[str, object]:
        endpoint = f"/wireguard/config/{wireguard_id}"

        return self._api_client.delete(endpoint)

    def wireguard_actions_generate_keys(self) -> dict[str, object]:
        endpoint = "/wireguard/actions/generate_keys"

        return self._api_client.post(endpoint)

    def get_wireguard_peers_config(self, wireguard_id: str) -> dict[str, object]:
        endpoint = f"/wireguard/{wireguard_id}/peers/config"

        return self._api_client.get(endpoint)

    def create_wireguard_peers_config(self, wireguard_id: str, config: dict[str, object]) -> dict[str, object]:
        endpoint = f"/wireguard/{wireguard_id}/peers/config"

        data = {"data": config}

        return self._api_client.post(endpoint, data=data)

    def update_wireguard_peers_config(self, wireguard_id: str, config: list[dict[str, object]]) -> dict[str, object]:
        endpoint = f"/wireguard/{wireguard_id}/peers/config"

        data = {"data": config}

        return self._api_client.put(endpoint, data=data)

    def delete_wireguard_peers_config(self, wireguard_id: str, config: list[str]) -> dict[str, object]:
        endpoint = f"/wireguard/{wireguard_id}/peers/config"

        data = {"data": config}

        return self._api_client.delete(endpoint, data=data)

    def get_wireguard_peer_config_by_id(self, wireguard_id: str, peer_id: str) -> dict[str, object]:
        endpoint = f"/wireguard/{wireguard_id}/peers/config/{peer_id}"

        return self._api_client.get(endpoint)

    def update_wireguard_peer_config_by_id(self, wireguard_id: str, peer_id: str, config: dict[str, object]) -> dict[str, object]:
        endpoint = f"/wireguard/{wireguard_id}/peers/config/{peer_id}"

        data = {"data": config}

        return self._api_client.put(endpoint, data=data)

    def delete_wireguard_peer_config_by_id(self, wireguard_id: str, peer_id: str) -> dict[str, object]:
        endpoint = f"/wireguard/{wireguard_id}/peers/config/{peer_id}"

        return self._api_client.delete(endpoint)



from .._client import APIClient


class Firmware:
    def __init__(self, api_client: APIClient) -> None:
        self._api_client = api_client

    def get_firmware_device_status(self) -> dict[str, object]:
        endpoint = "/firmware/device/status"

        return self._api_client.get(endpoint)

    def get_firmware_device_progress_status(self) -> dict[str, object]:
        endpoint = "/firmware/device/progress/status"

        return self._api_client.get(endpoint)

    def get_firmware_device_updates_status(self) -> dict[str, object]:
        endpoint = "/firmware/device/updates/status"

        return self._api_client.get(endpoint)

    def firmware_actions_fota_download(self) -> dict[str, object]:
        endpoint = "/firmware/actions/fota_download"

        return self._api_client.post(endpoint)

    def firmware_actions_upgrade(self, config: dict[str, object]) -> dict[str, object]:
        endpoint = "/firmware/actions/upgrade"

        data = {"data": config}

        return self._api_client.post(endpoint, data=data)



from .router import Router
from .rut200 import RUT200
from .rut240 import RUT240
from .rut241 import RUT241
from .rut956 import RUT956
from .rutx08 import RUTX08
from .rutx50 import RUTX50

__all__ = [
    "Router",
    "RUT200",
    "RUT240",
    "RUT241",
    "RUT956",
    "RUTX08",
    "RUTX50",
]



from ..._client import APIClient


class Gateway():
    def __init__(self, base_url: str) -> None:
        self._client = APIClient(base_url)

        self._endpoints = []

    def __getattr__(self, attr: str):
        for endpoint in self._endpoints:
            if hasattr(endpoint, attr):
                return getattr(endpoint, attr)

        raise AttributeError(
            f"'{self.__class__.__name__}' object has no attribute '{attr}'"
        )



from .._client import APIClient


class Firewall:
    def __init__(self, api_client: APIClient) -> None:
        self._api_client = api_client

    def get_firewall_connections_status(self) -> dict[str, object]:
        endpoint = "/firewall/connections/status"

        return self._api_client.get(endpoint)

    def get_firewall_port_forwards_config(self) -> dict[str, object]:
        endpoint = "/firewall/port_forwards/config"

        return self._api_client.get(endpoint)

    def create_firewall_port_forwards_config(self, config: dict[str, object]) -> dict[str, object]:
        endpoint = "/firewall/port_forwards/config"

        data = {"data": config}

        return self._api_client.post(endpoint, data=data)

    def update_firewall_port_forwards_config(self, config: list[dict[str, object]]) -> dict[str, object]:
        endpoint = "/firewall/port_forwards/config"

        data = {"data": config}

        return self._api_client.put(endpoint, data=data)

    def delete_firewall_port_forwards_config(self, config: list[str]) -> dict[str, object]:
        endpoint = "/firewall/port_forwards/config"

        data = {"data": config}

        return self._api_client.delete(endpoint, data=data)

    def get_firewall_port_forwards_config_by_id(self, port_forward_id: str) -> dict[str, object]:
        endpoint = f"/firewall/port_forwards/config/{port_forward_id}"

        return self._api_client.get(endpoint)

    def update_firewall_port_forwards_config_by_id(self, port_forward_id: str, config: dict[str, object]) -> dict[str, object]:
        endpoint = f"/firewall/port_forwards/config/{port_forward_id}"

        data = {"data": config}

        return self._api_client.put(endpoint, data=data)

    def delete_firewall_port_forwards_config_by_id(self, port_forward_id: str) -> dict[str, object]:
        endpoint = f"/firewall/port_forwards/config/{port_forward_id}"

        return self._api_client.delete(endpoint)
